/*****************************************************************************
 *  Rohde&Schwarz CMW500 Wideband Radio Communication Tester instrument driver
 *  Audio Measurement
 *  VXIpnp Instrument Driver
 *****************************************************************************/

#ifndef __RSCMWAM_HEADER
#define __RSCMWAM_HEADER

#include "rscore.h"
#include "rscmwam_attributes.h"
#include "rscmwam_utility.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <limits.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/****************************************************************************
 *----------------- Instrument Driver Revision Information -----------------*
 ****************************************************************************/

/* Class Driver Identification */

#define RSCMWAM_CLASS_DRIVER_DESCRIPTION           ""
#define RSCMWAM_CLASS_DRIVER_PREFIX                ""
#define RSCMWAM_CLASS_DRIVER_VENDOR                ""
#define RSCMWAM_CLASS_DRIVER_REVISION              ""
#define RSCMWAM_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSCMWAM_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSCMWAM_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSCMWAM_SPECIFIC_DRIVER_PREFIX             "RSCMWAM"
#define RSCMWAM_SPECIFIC_DRIVER_LOCATOR            ""
#define RSCMWAM_SPECIFIC_DRIVER_VENDOR             ""
#define RSCMWAM_MAJOR_VERSION                      3L    /* Instrument driver major version          */
#define RSCMWAM_MINOR_VERSION                      7L    /* Instrument driver minor version          */
#define RSCMWAM_MINOR_MINOR_VERSION                200L   /* Instrument driver minor minor version    */

/* Driver Capabilities */

#define RSCMWAM_SUPPORTED_INSTRUMENT_MODELS        ""    /* Instrument driver supported model(s)     */
#define RSCMWAM_GROUP_CAPABILITIES                 ""
#define RSCMWAM_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *------------------------------ Useful Macros -----------------------------*
 ****************************************************************************/

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/

#define RSCMWAM_SIMULATION_ID_QUERY      "Rohde&Schwarz,CMW,123456/789,1.23"  /* Default definition of ID Query Response for simulation */
#define RSCMWAM_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSCMWAM_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSCMWAM_OPC_TIMEOUT              5000L   /* Maximum time to wait for OPC in milliseconds */

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

    /*- Inherent Instrument Specific Attributes ----------------------------*/

        /* User Options */

#define RSCMWAM_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSCMWAM_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSCMWAM_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSCMWAM_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSCMWAM_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSCMWAM_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSCMWAM_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSCMWAM_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSCMWAM_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSCMWAM_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSCMWAM_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSCMWAM_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSCMWAM_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSCMWAM_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSCMWAM_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSCMWAM_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSCMWAM_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSCMWAM_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSCMWAM_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSCMWAM_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSCMWAM_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSCMWAM_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSCMWAM_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSCMWAM_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSCMWAM_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSCMWAM_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSCMWAM_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSCMWAM_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSCMWAM_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSCMWAM_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSCMWAM_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME                                    /* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

/* BEGIN GENERATE */
#define RSCMWAM_ATTR_ACTIVATE_STEREO_EXTERNAL_ANALOG_SPEECH_ANALYSIS        RS_SPECIFIC_PUBLIC_ATTR_BASE + 160
#define RSCMWAM_ATTR_ACTIVATE_STEREO_EXTERNAL_DIGITAL_SPEECH_ANALYSIS       RS_SPECIFIC_PUBLIC_ATTR_BASE + 161
#define RSCMWAM_ATTR_AUDIO_MEASUREMENT_AND_GENERATOR                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 154
#define RSCMWAM_ATTR_EXTERNAL_ANALOG_SPEECH_ANALYSIS                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 155
#define RSCMWAM_ATTR_EXTERNAL_DIGITAL_SPEECH_ANALYSIS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 156
#define RSCMWAM_ATTR_MICROPHONE_AND_SPEAKER_TEST                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 157
#define RSCMWAM_ATTR_WAVEFORM_TEST                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 177
#define RSCMWAM_ATTR_POSSIBLE_MASTER_STRINGS                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 158
#define RSCMWAM_ATTR_CLOCK_DRIFT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 159
#define RSCMWAM_ATTR_AUDIO_MEASUREMENT_REPETITION_MODE                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 116
#define RSCMWAM_ATTR_AUDIO_MEASUREMENT_STOP_CONDITION                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 117
#define RSCMWAM_ATTR_AUDIO_MEASUREMENT_SIGNAL_TYPE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 118
#define RSCMWAM_ATTR_AUDIO_MEASUREMENT_ANALOG_PATH_COUPLING                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 119
#define RSCMWAM_ATTR_AUDIO_MEASUREMENT_ANALOG_AUTO_RANGING                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 120
#define RSCMWAM_ATTR_AUDIO_MEASUREMENT_ANALOG_MAXIMUM_LEVEL                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 121
#define RSCMWAM_ATTR_SINGLE_TONE_STATISTIC_COUNT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 122
#define RSCMWAM_ATTR_SINGLE_TONE_FREQUENCY_DISTORTION                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 123
#define RSCMWAM_ATTR_SINGLE_TONE_LOW_PASS_FILTER                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 124
#define RSCMWAM_ATTR_SINGLE_TONE_HIGH_PASS_FILTER                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 125
#define RSCMWAM_ATTR_SINGLE_TONE_WEIGHTING_FILTER                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 126
#define RSCMWAM_ATTR_SINGLE_TONE_WINDOW_FUNCTION                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 150
#define RSCMWAM_ATTR_SINGLE_TONE_SPECTRUM_STATE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 151
#define RSCMWAM_ATTR_MULTITONE_STATISTIC_COUNT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 127
#define RSCMWAM_ATTR_MULTITONE_RESULT_MODE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 128
#define RSCMWAM_ATTR_MULTITONE_LOW_PASS_FILTER                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 129
#define RSCMWAM_ATTR_MULTITONE_HIGH_PASS_FILTER                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 130
#define RSCMWAM_ATTR_MULTITONE_WEIGHTING_FILTER                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 131
#define RSCMWAM_ATTR_FFT_NOISE_STATISTIC_COUNT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 132
#define RSCMWAM_ATTR_FFT_NOISE_LOW_PASS_FILTER                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 133
#define RSCMWAM_ATTR_FFT_NOISE_HIGH_PASS_FILTER                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 134
#define RSCMWAM_ATTR_FFT_NOISE_WEIGHTING_FILTER                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 135
#define RSCMWAM_ATTR_AUDIO_MEASUREMENT_INIT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 136
#define RSCMWAM_ATTR_AUDIO_MEASUREMENT_ABORT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 137
#define RSCMWAM_ATTR_AUDIO_MEASUREMENT_STOP                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 138
#define RSCMWAM_ATTR_GENERATOR_SIGNAL_TYPE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 139
#define RSCMWAM_ATTR_GENERATOR_ANALYZER_COUPLING                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 140
#define RSCMWAM_ATTR_GENERATOR_SINGLE_TONE_FREQUENCY                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 141
#define RSCMWAM_ATTR_GENERATOR_SINGLE_TONE_LEVEL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 142
#define RSCMWAM_ATTR_GENERATOR_MULTITONE_CREST_FACTOR                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 143
#define RSCMWAM_ATTR_GENERATOR_MULTITONE_LEVEL_SELECTION                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 144
#define RSCMWAM_ATTR_GENERATOR_MULTITONE_TOTAL_LEVEL                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 145
#define RSCMWAM_ATTR_QUERY_GENERATOR_MULTITONE_TOTAL_LEVEL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 146
#define RSCMWAM_ATTR_GENERATOR_MULTITONE_TONE_START                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 147
#define RSCMWAM_ATTR_GENERATOR_MULTITONE_TONE_STOP                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 148
#define RSCMWAM_ATTR_GENERATOR_FFT_NOISE_LEVEL                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 149
#define RSCMWAM_ATTR_SPEECH_ANALYSIS_INPUT_LEVEL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 152
#define RSCMWAM_ATTR_SPEECH_ANALYSIS_OUTPUT_LEVEL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 153
#define RSCMWAM_ATTR_SPEECH_ANALYSIS_FILTER_HIGH_PASS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 162
#define RSCMWAM_ATTR_RECORDING_INITIATE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 163
#define RSCMWAM_ATTR_RECORDING_ABORT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 164
#define RSCMWAM_ATTR_RECORDING_STOP                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 165
#define RSCMWAM_ATTR_RECORDING_AMOUNT_OF_RECORDED_DATA                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 166
#define RSCMWAM_ATTR_RECORDING_INPUT_LEVEL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 168
#define RSCMWAM_ATTR_RECORDING_HIGHPASS_FILTER_FREQUENCY                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 169
#define RSCMWAM_ATTR_RECORDING_FILE_PATH_NAME                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 170
#define RSCMWAM_ATTR_PLAYBACK_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 171
#define RSCMWAM_ATTR_PLAYBACK_STATE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 172
#define RSCMWAM_ATTR_PLAYBACK_OUTPUT_LEVEL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 174
#define RSCMWAM_ATTR_PLAYBACK_FILE_PATH_NAME                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 175
#define RSCMWAM_ATTR_PLAYBACK_PROGRESS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 176
#define RSCMWAM_ATTR_SYSTEM_VERSION                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 39
#define RSCMWAM_ATTR_SYSTEM_ERROR                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 40
#define RSCMWAM_ATTR_SYSTEM_ALL_ERRORS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 41
#define RSCMWAM_ATTR_SYSTEM_PRESET                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 42
#define RSCMWAM_ATTR_ID_QUERY_RESPONSE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 43

// rscmwam_rngAudioMeasurementRepetitionMode
#define RSCMWAM_VAL_REPETITION_SINGLE                                       0
#define RSCMWAM_VAL_REPETITION_CONT                                         1

// rscmwam_rngAudioMeasurementStopCondition
#define RSCMWAM_VAL_NONE                                                    0
#define RSCMWAM_VAL_LIM_FAIL                                                1

// rscmwam_rngAudioMeasurementSignalType
#define RSCMWAM_VAL_SINGLE_TONE                                             0
#define RSCMWAM_VAL_MULTITONE                                               1
#define RSCMWAM_VAL_FFT_NOISE                                               2

// rscmwam_rngAudioMeasurementAnalogPathCoupling
#define RSCMWAM_VAL_AC                                                      0
#define RSCMWAM_VAL_DC                                                      1

// rscmwam_rngAudioMeasurementAnalogAutoRanging
#define RSCMWAM_VAL_OFF                                                     0
#define RSCMWAM_VAL_ON                                                      1

// rscmwam_rngSingleToneLowPass
#define RSCMWAM_VAL_SINGLE_TONE_LOW_PASS_FILTER_OFF                         0
#define RSCMWAM_VAL_SINGLE_TONE_LOW_PASS_FILTER_LP3                         1
#define RSCMWAM_VAL_SINGLE_TONE_LOW_PASS_FILTER_LP4                         2
#define RSCMWAM_VAL_SINGLE_TONE_LOW_PASS_FILTER_LP15                        3

// rscmwam_rngSingleToneHighPass
#define RSCMWAM_VAL_SINGLE_TONE_HIGH_PASS_FILTER_OFF                        0
#define RSCMWAM_VAL_SINGLE_TONE_HIGH_PASS_FILTER_H6                         1
#define RSCMWAM_VAL_SINGLE_TONE_HIGH_PASS_FILTER_H50                        2
#define RSCMWAM_VAL_SINGLE_TONE_HIGH_PASS_FILTER_H300                       3

// rscmwam_rngSingleToneWeightingFilter
#define RSCMWAM_VAL_SINGLE_TONE_WEIGHTING_FILTER_OFF                        0
#define RSCMWAM_VAL_SINGLE_TONE_WEIGHTING_FILTER_AWE                        1
#define RSCMWAM_VAL_SINGLE_TONE_WEIGHTING_FILTER_CCIT                       2
#define RSCMWAM_VAL_SINGLE_TONE_WEIGHTING_FILTER_CMSG                       3

// rscmwam_rngMultitoneResultMode
#define RSCMWAM_VAL_RESULT_MODE_ABSOLUTE_LEVEL                              0
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_01                            1
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_02                            2
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_03                            3
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_04                            4
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_05                            5
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_06                            6
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_07                            7
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_08                            8
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_09                            9
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_10                            10
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_11                            11
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_12                            12
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_13                            13
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_14                            14
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_15                            15
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_16                            16
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_17                            17
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_18                            18
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_19                            19
#define RSCMWAM_VAL_RESULT_MODE_RELATIVE_TONE_20                            20

// rscmwam_rngMultitoneLowPassFilter
#define RSCMWAM_VAL_MULTITONE_LOW_PASS_FILTER_OFF                           0
#define RSCMWAM_VAL_MULTITONE_LOW_PASS_FILTER_LP3                           1
#define RSCMWAM_VAL_MULTITONE_LOW_PASS_FILTER_LP4                           2
#define RSCMWAM_VAL_MULTITONE_LOW_PASS_FILTER_LP15                          3

// rscmwam_rngMultitoneHighPassFilter
#define RSCMWAM_VAL_MULTITONE_HIGH_PASS_FILTER_OFF                          0
#define RSCMWAM_VAL_MULTITONE_HIGH_PASS_FILTER_H6                           1
#define RSCMWAM_VAL_MULTITONE_HIGH_PASS_FILTER_H50                          2
#define RSCMWAM_VAL_MULTITONE_HIGH_PASS_FILTER_H300                         3

// rscmwam_rngMultitoneWeightingFilter
#define RSCMWAM_VAL_MULTITONE_WEIGHTING_FILTER_OFF                          0
#define RSCMWAM_VAL_MULTITONE_WEIGHTING_FILTER_AWE                          1
#define RSCMWAM_VAL_MULTITONE_WEIGHTING_FILTER_CCIT                         2
#define RSCMWAM_VAL_MULTITONE_WEIGHTING_FILTER_CMSG                         3

// rscmwam_rngFFTNoiseLowPassFilter
#define RSCMWAM_VAL_FFT_NOISE_LOW_PASS_FILTER_OFF                           0
#define RSCMWAM_VAL_FFT_NOISE_LOW_PASS_FILTER_LP3                           1
#define RSCMWAM_VAL_FFT_NOISE_LOW_PASS_FILTER_LP4                           2
#define RSCMWAM_VAL_FFT_NOISE_LOW_PASS_FILTER_LP15                          3

// rscmwam_rngFFTNoiseHighPassFilter
#define RSCMWAM_VAL_FFT_NOISE_HIGH_PASS_FILTER_OFF                          0
#define RSCMWAM_VAL_FFT_NOISE_HIGH_PASS_FILTER_H6                           1
#define RSCMWAM_VAL_FFT_NOISE_HIGH_PASS_FILTER_H50                          2
#define RSCMWAM_VAL_FFT_NOISE_HIGH_PASS_FILTER_H300                         3

// rscmwam_rngFFTNoiseWeightingFilter
#define RSCMWAM_VAL_FFT_NOISE_WEIGHTING_FILTER_OFF                          0
#define RSCMWAM_VAL_FFT_NOISE_WEIGHTING_FILTER_AWE                          1
#define RSCMWAM_VAL_FFT_NOISE_WEIGHTING_FILTER_CCIT                         2
#define RSCMWAM_VAL_FFT_NOISE_WEIGHTING_FILTER_CMSG                         3

// rscmwam_rngGeneratorSignalType
#define RSCMWAM_VAL_SIGNAL_TYPE_SINGLE_TONE                                 0
#define RSCMWAM_VAL_SIGNAL_TYPE_MULTITONE                                   1
#define RSCMWAM_VAL_SIGNAL_TYPE_FFT_NOISE                                   2

// rscmwam_rngGeneratorAnalyzerCoupling
#define RSCMWAM_VAL_ANALYZER_COUPLING_OFF                                   0
#define RSCMWAM_VAL_ANALYZER_COUPLING_ANALOG                                1
#define RSCMWAM_VAL_ANALYZER_COUPLING_DIGITAL                               2

// rscmwam_rngGeneratorMultitoneCrestFactor
#define RSCMWAM_VAL_CREST_MAX                                               0
#define RSCMWAM_VAL_CREST_LOW                                               1

// rscmwam_rngGeneratorMultitoneLevelSelection
#define RSCMWAM_VAL_LEVEL_TOTAL_LEVEL                                       0
#define RSCMWAM_VAL_LEVEL_SEPARATELY_LEVEL                                  1

// rscmwam_rngSingleToneWindowFunction
#define RSCMWAM_VAL_SINGLE_TONE_WINDOW_FUNCTION_RECT                        0
#define RSCMWAM_VAL_SINGLE_TONE_WINDOW_FUNCTION_HAMM                        1
#define RSCMWAM_VAL_SINGLE_TONE_WINDOW_FUNCTION_HANN                        2
#define RSCMWAM_VAL_SINGLE_TONE_WINDOW_FUNCTION_BHAR                        3
#define RSCMWAM_VAL_SINGLE_TONE_WINDOW_FUNCTION_FTOP                        4

// rscmwam_rngFilterHighPass
#define RSCMWAM_VAL_FILTER_OFF                                              0
#define RSCMWAM_VAL_FILTER_H6                                               1
#define RSCMWAM_VAL_FILTER_H50                                              2
#define RSCMWAM_VAL_FILTER_H300                                             3

// rscmwam_rngDNDRecSource
#define RSCMWAM_VAL_REC_SOURCE_ANALOG                                       0
#define RSCMWAM_VAL_REC_SOURCE_DIGITAL                                      1
#define RSCMWAM_VAL_REC_SOURCE_SPEECH                                       2

// rscmwam_rngPlaybackState
#define RSCMWAM_VAL_PLAYBACK_STATE_OFF                                      0
#define RSCMWAM_VAL_PLAYBACK_STATE_PEND                                     1
#define RSCMWAM_VAL_PLAYBACK_STATE_ON                                       2
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSCMWAM_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSCMWAM_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSCMWAM_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSCMWAM_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

/****************************************************************************
 *------------------------ Attribute Value Defines -------------------------*
 ****************************************************************************/

    /*- Instrument specific attribute value definitions --------------------*/

#define RSCMWAM_VAL_ANALOG    0
#define RSCMWAM_VAL_DIGITAL   1

#define RSCMWAM_VAL_CUTOFF_FREQUENCY_OFF    0
#define RSCMWAM_VAL_CUTOFF_FREQUENCY_LP3    1
#define RSCMWAM_VAL_CUTOFF_FREQUENCY_LP4    2
#define RSCMWAM_VAL_CUTOFF_FREQUENCY_LP15   3

#define RSCMWAM_VAL_CUTOFF_FREQUENCY_OFF    0
#define RSCMWAM_VAL_CUTOFF_FREQUENCY_H6     1
#define RSCMWAM_VAL_CUTOFF_FREQUENCY_H50    2
#define RSCMWAM_VAL_CUTOFF_FREQUENCY_H300   3

#define RSCMWAM_VAL_WEIGHTING_FILTER_OFF    0
#define RSCMWAM_VAL_WEIGHTING_FILTER_AWE    1
#define RSCMWAM_VAL_WEIGHTING_FILTER_CCIT   2
#define RSCMWAM_VAL_WEIGHTING_FILTER_CMSG   3

#define RSCMWAM_VAL_RESULT_CURRENT          0
#define RSCMWAM_VAL_RESULT_AVERAGE          1
#define RSCMWAM_VAL_RESULT_MAX              2
#define RSCMWAM_VAL_RESULT_MIN              3
#define RSCMWAM_VAL_RESULT_SDEV             4
#define RSCMWAM_VAL_RESULT_EXTREME          5

#define RSCMWAM_VAL_LIMIT_LOWER             0
#define RSCMWAM_VAL_LIMIT_UPPER             1

#define RSCMWAM_VAL_SCENARIO_AUDIO_MEAS_GEN     1
#define RSCMWAM_VAL_SCENARIO_EXT_ANALOG_SPEECH  2
#define RSCMWAM_VAL_SCENARIO_EXT_DIGITAL_SPEECH 3
#define RSCMWAM_VAL_SCENARIO_EXT_ANALOG_STEREO  4
#define RSCMWAM_VAL_SCENARIO_EXT_DIGITAL_STEREO 5
#define RSCMWAM_VAL_SCENARIO_MIC_SPEAKER_TEST   6
#define RSCMWAM_VAL_SCENARIO_WAVEFORM_TEST      7

/****************************************************************************
 *---------------- Instrument Driver Function Declarations -----------------*
 ****************************************************************************/
ViStatus _VI_FUNC rscmwam_init (ViRsrc resourceName, ViBoolean IDQuery,
                                ViBoolean resetDevice, ViSession* instrumentHandle);
ViStatus _VI_FUNC rscmwam_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                                           ViBoolean resetDevice,
                                           ViString optionString,
                                           ViSession* instrumentHandle);
ViStatus _VI_FUNC rscmwam_ActivateAudioScenario (ViSession instrumentHandle,
                                                 ViInt32 instance, ViInt32 scenario,
                                                 ViString master);
ViStatus _VI_FUNC rscmwam_ConfigureAudioMeasurementandGenerator
             (ViSession instrumentHandle, ViInt32 instance, ViString master);
ViStatus _VI_FUNC rscmwam_ConfigureExternalAnalogSpeechAnalysis
             (ViSession instrumentHandle, ViInt32 instance, ViString master);
ViStatus _VI_FUNC rscmwam_ConfigureExternalDigitalSpeechAnalysis
             (ViSession instrumentHandle, ViInt32 instance, ViString master);
ViStatus _VI_FUNC rscmwam_ConfigureMicrophoneandSpeakerTest
             (ViSession instrumentHandle, ViInt32 instance,
              ViString masterApplication);
ViStatus _VI_FUNC rscmwam_QueryPossibleMasterStrings (ViSession instrumentHandle,
                                                      ViInt32 instance,
                                                      ViInt32 arraySize,
                                                      ViChar _VI_FAR masterStrings[]);
ViStatus _VI_FUNC rscmwam_ConfigureClockDrift (ViSession instrumentHandle,
                                               ViInt32 instance,
                                               ViReal64 clockDrift);
ViStatus _VI_FUNC rscmwam_ConfigureAudioMeasurementStatisticsSettings
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 repetition, ViInt32 stopCondition);
ViStatus _VI_FUNC rscmwam_ConfigureAudioMeasurementStatisticsCounter
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 singleTone, ViInt32 multitone, ViInt32 FFTNoise);
ViStatus _VI_FUNC rscmwam_QueryAudioMeasurementSignalType
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32* signalType);
ViStatus _VI_FUNC rscmwam_ConfigureAudioMeasurementAnalogInputSettings
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 pathCoupling,
              ViInt32 autoRanging, ViReal64 maximumLevel);
ViStatus _VI_FUNC rscmwam_ConfigureAudioMeasurementLowpassFilter
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 frequencySingleTone, ViInt32 frequencyMultitone,
              ViInt32 frequencyFFTNoise);
ViStatus _VI_FUNC rscmwam_ConfigureAudioMeasurementHighpassFilter
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 frequencySingleTone, ViInt32 frequencyMultitone,
              ViInt32 frequencyFFTNoise);
ViStatus _VI_FUNC rscmwam_ConfigureAudioMeasurementWeightingFilter
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 singleTone, ViInt32 multitone, ViInt32 FFTNoise);
ViStatus _VI_FUNC rscmwam_AudioMeasurementInit (ViSession instrumentHandle,
                                                ViInt32 instance,
                                                ViInt32 audioType);
ViStatus _VI_FUNC rscmwam_AudioMeasurementAbort (ViSession instrumentHandle,
                                                 ViInt32 instance,
                                                 ViInt32 audioType);
ViStatus _VI_FUNC rscmwam_AudioMeasurementStop (ViSession instrumentHandle,
                                                ViInt32 instance,
                                                ViInt32 audioType);
ViStatus _VI_FUNC rscmwam_QueryAudioMeasurementStatus (ViSession instrumentHandle,
                                                       ViInt32 instance,
                                                       ViInt32 audioType,
                                                       ViInt32* mainState,
                                                       ViInt32* syncState,
                                                       ViInt32* resourcesState);
ViStatus _VI_FUNC rscmwam_ConfigureSingleToneMeasurementStatisticsCount
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 statisticCount);
ViStatus _VI_FUNC rscmwam_ConfigureSingleToneMeasurementFilterSettings
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViReal64 distorsionFrequency, ViInt32 lowPass, ViInt32 highPass,
              ViInt32 weighting);
ViStatus _VI_FUNC rscmwam_ConfigureSingleToneMeasurementFilterWindowFunction
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 windowFunction);
ViStatus _VI_FUNC rscmwam_ConfigureSingleToneMeasurementFilterBandPass
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViReal64 centerFrequency, ViReal64 bandwidth);
ViStatus _VI_FUNC rscmwam_ConfigureSingleToneAnalogMeasurementLimits
             (ViSession instrumentHandle, ViInt32 instance, ViReal64 weightRMSUpper,
              ViBoolean weightRMSEnabled, ViReal64 bandRMSUpper,
              ViBoolean bandRMSEnabled, ViReal64 THDPercentUpper,
              ViBoolean THDPercentEnabled, ViReal64 THDNUpper,
              ViBoolean THDNEnabled, ViReal64 SNRLow, ViBoolean SNREnabled);
ViStatus _VI_FUNC rscmwam_ConfigureSingleToneDigitalMeasurementLimits
             (ViSession instrumentHandle, ViInt32 instance, ViReal64 weightRMSUpper,
              ViBoolean weightRMSEnabled, ViReal64 bandRMSUpper,
              ViBoolean bandRMSEnabled, ViReal64 THDPercentUpper,
              ViBoolean THDPercentEnabled, ViReal64 THDNUpper,
              ViBoolean THDNEnabled, ViReal64 SNRLow, ViBoolean SNREnabled);
ViStatus _VI_FUNC rscmwam_ConfigureSingleToneSpectrumState
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViBoolean spectrumState);
ViStatus _VI_FUNC rscmwam_ReadSingleToneAnalogMeasurementLimits
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 resultType,
              ViInt32 arraySize, ViInt32* reliability, ViReal64 _VI_FAR results[],
              ViInt32* returnedValues);
ViStatus _VI_FUNC rscmwam_FetchSingleToneAnalogMeasurementLimits
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 resultType,
              ViInt32 arraySize, ViInt32* reliability, ViReal64 _VI_FAR results[],
              ViInt32* returnedValues);
ViStatus _VI_FUNC rscmwam_ReadSingleToneDigitalMeasurementLimits
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 resultType,
              ViInt32 arraySize, ViInt32* reliability, ViReal64 _VI_FAR results[],
              ViInt32* returnedValues);
ViStatus _VI_FUNC rscmwam_FetchSingleToneDigitalMeasurementLimits
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 resultType,
              ViInt32 arraySize, ViInt32* reliability, ViReal64 _VI_FAR results[],
              ViInt32* returnedValues);
ViStatus _VI_FUNC rscmwam_ConfigureMultitoneMeasurementStatisticsCount
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 statisticCount);
ViStatus _VI_FUNC rscmwam_ConfigureMultitoneMeasurementResultMode
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 resultMode);
ViStatus _VI_FUNC rscmwam_ConfigureMultitoneMeasurementFilterSettings
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 lowPass, ViInt32 highPass, ViInt32 weighting);
ViStatus _VI_FUNC rscmwam_ConfigureMultitoneAnalogMeasurementLimitsList
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 limitType,
              ViInt32 arraySize, ViReal64 _VI_FAR limits[],
              ViBoolean _VI_FAR states[]);
ViStatus _VI_FUNC rscmwam_ConfigureMultitoneAnalogMeasurementLimits
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 tone,
              ViInt32 limitType, ViReal64 limit, ViBoolean state);
ViStatus _VI_FUNC rscmwam_ConfigureMultitoneDigitalMeasurementLimitsList
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 limitType,
              ViInt32 arraySize, ViReal64 _VI_FAR limits[],
              ViBoolean _VI_FAR states[]);
ViStatus _VI_FUNC rscmwam_ConfigureMultitoneDigitalMeasurementLimits
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 tone,
              ViInt32 limitType, ViReal64 limit, ViBoolean state);
ViStatus _VI_FUNC rscmwam_ReadMultitoneAnalogMeasurementLimits
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 resultType,
              ViInt32 arraySize, ViInt32* reliability, ViReal64 _VI_FAR results[],
              ViInt32* returnedValues);
ViStatus _VI_FUNC rscmwam_FetchMultitoneAnalogMeasurementLimits
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 resultType,
              ViInt32 arraySize, ViInt32* reliability, ViReal64 _VI_FAR results[],
              ViInt32* returnedValues);
ViStatus _VI_FUNC rscmwam_ReadMultitoneDigitalMeasurementLimits
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 resultType,
              ViInt32 arraySize, ViInt32* reliability, ViReal64 _VI_FAR results[],
              ViInt32* returnedValues);
ViStatus _VI_FUNC rscmwam_FetchMultitoneDigitalMeasurementLimits
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 resultType,
              ViInt32 arraySize, ViInt32* reliability, ViReal64 _VI_FAR results[],
              ViInt32* returnedValues);
ViStatus _VI_FUNC rscmwam_ConfigureFFTNoiseMeasurementStatisticsCount
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 statisticCount);
ViStatus _VI_FUNC rscmwam_ConfigureFFTNoiseMeasurementFilterSettings
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 lowPass, ViInt32 highPass, ViInt32 weighting);
ViStatus _VI_FUNC rscmwam_ReadFFTNoiseAnalogMeasurementLevelResults
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 resultType,
              ViInt32 arraySize, ViInt32* reliability, ViReal64 _VI_FAR level[],
              ViInt32* returnedValues);
ViStatus _VI_FUNC rscmwam_FetchFFTNoiseAnalogMeasurementLevelResults
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 resultType,
              ViInt32 arraySize, ViInt32* reliability, ViReal64 _VI_FAR level[],
              ViInt32* returnedValues);
ViStatus _VI_FUNC rscmwam_ReadFFTNoiseAnalogMeasurementFrequencyResults
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 arraySize,
              ViInt32* reliability, ViReal64 _VI_FAR frequency[],
              ViInt32* returnedValues);
ViStatus _VI_FUNC rscmwam_FetchFFTNoiseAnalogMeasurementFrequencyResults
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 arraySize,
              ViInt32* reliability, ViReal64 _VI_FAR frequency[],
              ViInt32* returnedValues);
ViStatus _VI_FUNC rscmwam_ReadFFTNoiseDigitalMeasurementLevelResults
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 resultType,
              ViInt32 arraySize, ViInt32* reliability, ViReal64 _VI_FAR level[],
              ViInt32* returnedValues);
ViStatus _VI_FUNC rscmwam_FetchFFTNoiseDigitalMeasurementLevelResults
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 resultType,
              ViInt32 arraySize, ViInt32* reliability, ViReal64 _VI_FAR level[],
              ViInt32* returnedValues);
ViStatus _VI_FUNC rscmwam_ReadFFTNoiseDigitalMeasurementFrequencyResults
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 arraySize,
              ViInt32* reliability, ViReal64 _VI_FAR frequency[],
              ViInt32* returnedValues);
ViStatus _VI_FUNC rscmwam_FetchFFTNoiseDigitalMeasurementFrequencyResults
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 arraySize,
              ViInt32* reliability, ViReal64 _VI_FAR frequency[],
              ViInt32* returnedValues);
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorSignalType (ViSession instrumentHandle,
                                                        ViInt32 instance,
                                                        ViInt32 audioType,
                                                        ViInt32 signalType);
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorAnalyzerCoupling
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 analyzerCoupling);
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorSingleToneList
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViReal64 frequency, ViReal64 level);
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorMultitoneCrestFactor
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 crestFactor);
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorMultitoneLevelSelection
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 levelSelection);
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorMultitoneTotalLevel
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViReal64 totalLevel);
ViStatus _VI_FUNC rscmwam_QueryGeneratorMultitoneTotalLevel
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViReal64* totalLevel);
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorMultitoneToneSettings
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 startTone, ViInt32 stopTone);
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorMultitoneDefinedToneList
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 tone, ViReal64 frequency, ViReal64 level);
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorMultitoneListAll
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViInt32 arraySize, ViReal64 _VI_FAR frequency[],
              ViReal64 _VI_FAR level[]);
ViStatus _VI_FUNC rscmwam_ConfigureGeneratorFFTNoiseLevel
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 audioType,
              ViReal64 level);
ViStatus _VI_FUNC rscmwam_QueryEventLogAllEntries (ViSession instrumentHandle,
                                                   ViInt32 instance,
                                                   ViInt32 arraySize,
                                                   ViInt32 bufferSize,
                                                   ViChar _VI_FAR timeStamp[],
                                                   ViInt32* eventCategory,
                                                   ViChar _VI_FAR message[],
                                                   ViInt32* numberOfEntries);
ViStatus _VI_FUNC rscmwam_QueryEventLogLastEntry (ViSession instrumentHandle,
                                                  ViInt32 instance,
                                                  ViInt32 bufferSize,
                                                  ViChar _VI_FAR timeStamp[],
                                                  ViInt32* eventCategory,
                                                  ViChar _VI_FAR message[]);
ViStatus _VI_FUNC rscmwam_ConfigureSpeechAnalysisControl (ViSession instrumentHandle,
                                                          ViInt32 instance,
                                                          ViReal64 inputLevelFullScale,
                                                          ViReal64 outputLevelFullScale);
ViStatus _VI_FUNC rscmwam_ConfigureSpeechAnalysisFilter (ViSession instrumentHandle,
                                                         ViInt32 instance,
                                                         ViInt32 highPass);
ViStatus _VI_FUNC rscmwam_RecordingInit (ViSession instrumentHandle,
                                         ViInt32 instance);
ViStatus _VI_FUNC rscmwam_RecordingAbort (ViSession instrumentHandle,
                                          ViInt32 instance);
ViStatus _VI_FUNC rscmwam_RecordingStop (ViSession instrumentHandle,
                                         ViInt32 instance);
ViStatus _VI_FUNC rscmwam_QueryRecordingAmountOfRecordedData
             (ViSession instrumentHandle, ViInt32 instance, ViReal64* recordedData);
ViStatus _VI_FUNC rscmwam_QueryRecordingStatus (ViSession instrumentHandle,
                                                ViInt32 instance,
                                                ViInt32* mainState,
                                                ViInt32* syncState,
                                                ViInt32* resourcesState);
ViStatus _VI_FUNC rscmwam_FetchRecordingStatus (ViSession instrumentHandle,
                                                ViInt32 instance,
                                                ViInt32* mainState);
ViStatus _VI_FUNC rscmwam_ConfigureRecordingSource (ViSession instrumentHandle,
                                                    ViInt32 instance,
                                                    ViInt32 source,
                                                    ViString master);
ViStatus _VI_FUNC rscmwam_ConfigureRecordingInputLevel (ViSession instrumentHandle,
                                                        ViInt32 instance,
                                                        ViReal64 inputLevel);
ViStatus _VI_FUNC rscmwam_ConfigureRecordingHighpassFilterFrequency
             (ViSession instrumentHandle, ViInt32 instance, ViInt32 frequency);
ViStatus _VI_FUNC rscmwam_ConfigureRecordingFilePathName (ViSession instrumentHandle,
                                                          ViInt32 instance,
                                                          ViString filePathName);
ViStatus _VI_FUNC rscmwam_ConfigurePlaybackEnabled (ViSession instrumentHandle,
                                                    ViInt32 instance,
                                                    ViBoolean enabled);
ViStatus _VI_FUNC rscmwam_QueryPlaybackState (ViSession instrumentHandle,
                                              ViInt32 instance, ViInt32* state);
ViStatus _VI_FUNC rscmwam_ConfigurePlaybackDestination (ViSession instrumentHandle,
                                                        ViInt32 instance,
                                                        ViInt32 source,
                                                        ViString master);
ViStatus _VI_FUNC rscmwam_ConfigurePlaybackOutputLevel (ViSession instrumentHandle,
                                                        ViInt32 instance,
                                                        ViReal64 outputLevel);
ViStatus _VI_FUNC rscmwam_ConfigurePlaybackFilePathName (ViSession instrumentHandle,
                                                         ViInt32 instance,
                                                         ViString filePathName);
ViStatus _VI_FUNC rscmwam_QueryPlaybackProgress (ViSession instrumentHandle,
                                                 ViInt32 instance,
                                                 ViInt32* progress);
ViStatus _VI_FUNC rscmwam_SetAttributeViInt32 (ViSession instrumentHandle,
                                               ViString repeatedCapabilityName,
                                               ViUInt32 attributeID,
                                               ViInt32 attributeValue);
ViStatus _VI_FUNC rscmwam_SetAttributeViReal64 (ViSession instrumentHandle,
                                                ViString repeatedCapabilityName,
                                                ViUInt32 attributeID,
                                                ViReal64 attributeValue);
ViStatus _VI_FUNC rscmwam_SetAttributeViString (ViSession instrumentHandle,
                                                ViString repeatedCapabilityName,
                                                ViUInt32 attributeID,
                                                ViString attributeValue);
ViStatus _VI_FUNC rscmwam_SetAttributeViBoolean (ViSession instrumentHandle,
                                                 ViString repeatedCapabilityName,
                                                 ViUInt32 attributeID,
                                                 ViBoolean attributeValue);
ViStatus _VI_FUNC rscmwam_SetAttributeViSession (ViSession instrumentHandle,
                                                 ViString repeatedCapabilityName,
                                                 ViUInt32 attributeID,
                                                 ViSession attributeValue);
ViStatus _VI_FUNC rscmwam_GetAttributeRepCapName (ViSession instrumentHandle,
                                                  ViUInt32 attributeID,
                                                  ViString repCapNameID,
                                                  ViInt32 index, ViInt32 bufferSize,
                                                  ViChar _VI_FAR repCapName[]);
ViStatus _VI_FUNC rscmwam_GetAttributeRepeatedCapabilityIds
             (ViSession instrumentHandle, ViUInt32 attributeID, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityId_s[]);
ViStatus _VI_FUNC rscmwam_GetAttributeRepeatedCapabilityIdNames
             (ViSession instrumentHandle, ViUInt32 attributeID,
              ViString repeatedCapabilityId, ViInt32 bufferSize,
              ViChar _VI_FAR repeatedCapabilityIdName_s[]);
ViStatus _VI_FUNC rscmwam_GetAttributeViInt32 (ViSession instrumentHandle,
                                               ViString repeatedCapabilityName,
                                               ViUInt32 attributeID,
                                               ViInt32* attributeValue);
ViStatus _VI_FUNC rscmwam_GetAttributeViReal64 (ViSession instrumentHandle,
                                                ViString repeatedCapabilityName,
                                                ViUInt32 attributeID,
                                                ViReal64* attributeValue);
ViStatus _VI_FUNC rscmwam_GetAttributeViString (ViSession instrumentHandle,
                                                ViString repeatedCapabilityName,
                                                ViUInt32 attributeID,
                                                ViInt32 bufferSize,
                                                ViChar _VI_FAR attributeValue[]);
ViStatus _VI_FUNC rscmwam_GetAttributeViBoolean (ViSession instrumentHandle,
                                                 ViString repeatedCapabilityName,
                                                 ViUInt32 attributeID,
                                                 ViBoolean* attributeValue);
ViStatus _VI_FUNC rscmwam_GetAttributeViSession (ViSession instrumentHandle,
                                                 ViString repeatedCapabilityName,
                                                 ViUInt32 attributeID,
                                                 ViSession* attributeValue);
ViStatus _VI_FUNC rscmwam_CheckAttributeViInt32 (ViSession instrumentHandle,
                                                 ViString repeatedCapabilityName,
                                                 ViUInt32 attributeID,
                                                 ViInt32 attributeValue);
ViStatus _VI_FUNC rscmwam_CheckAttributeViReal64 (ViSession instrumentHandle,
                                                  ViString repeatedCapabilityName,
                                                  ViUInt32 attributeID,
                                                  ViReal64 attributeValue);
ViStatus _VI_FUNC rscmwam_CheckAttributeViString (ViSession instrumentHandle,
                                                  ViString repeatedCapabilityName,
                                                  ViUInt32 attributeID,
                                                  ViString attributeValue);
ViStatus _VI_FUNC rscmwam_CheckAttributeViBoolean (ViSession instrumentHandle,
                                                   ViString repeatedCapabilityName,
                                                   ViUInt32 attributeID,
                                                   ViBoolean attributeValue);
ViStatus _VI_FUNC rscmwam_CheckAttributeViSession (ViSession instrumentHandle,
                                                   ViString repeatedCapabilityName,
                                                   ViUInt32 attributeID,
                                                   ViSession attributeValue);
ViStatus _VI_FUNC rscmwam_reset (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmwam_self_test (ViSession instrumentHandle,
                                     ViInt16* selfTestResult,
                                     ViChar _VI_FAR selfTestMessage[]);
ViStatus _VI_FUNC rscmwam_revision_query (ViSession instrumentHandle,
                                          ViChar _VI_FAR instrumentDriverRevision[],
                                          ViChar _VI_FAR firmwareRevision[]);
ViStatus _VI_FUNC rscmwam_SystemVersion (ViSession instrumentHandle, ViInt32 length,
                                         ViChar _VI_FAR systemVersion[]);
ViStatus _VI_FUNC rscmwam_error_query (ViSession instrumentHandle,
                                       ViInt32* errorCode,
                                       ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rscmwam_ErrorList (ViSession instrumentHandle, ViInt32 bufferSize,
                                     ViChar _VI_FAR errors[]);
ViStatus _VI_FUNC rscmwam_error_message (ViSession instrumentHandle,
                                         ViStatus errorCode,
                                         ViChar _VI_FAR errorMessage[]);
ViStatus _VI_FUNC rscmwam_ReliabilityIndicator (ViSession instrumentHandle,
                                                ViInt32 reliabilityCode,
                                                ViChar _VI_FAR reliabilityMessage[]);
ViStatus _VI_FUNC rscmwam_Preset (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmwam_ClearStatus (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmwam_IDQueryResponse (ViSession instrumentHandle,
                                           ViChar _VI_FAR IDQueryResponse[]);
ViStatus _VI_FUNC rscmwam_ProcessAllPreviousCommands (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmwam_QueryOPC (ViSession instrumentHandle, ViInt32* OPC);
ViStatus _VI_FUNC rscmwam_SetVISATimeout (ViSession instrumentHandle,
                                          ViUInt32 VISATimeout);
ViStatus _VI_FUNC rscmwam_GetVISATimeout (ViSession instrumentHandle,
                                          ViUInt32* VISATimeout);
ViStatus _VI_FUNC rscmwam_ConfigureErrorChecking (ViSession instrumentHandle,
                                                  ViBoolean optionChecking,
                                                  ViBoolean rangeChecking,
                                                  ViBoolean statusChecking);
ViStatus _VI_FUNC rscmwam_ConfigureAutoSystemErrQuery (ViSession instrumentHandle,
                                                       ViBoolean autoSystErrQuery);
ViStatus _VI_FUNC rscmwam_ConfigureMultiThreadLocking (ViSession instrumentHandle,
                                                       ViBoolean multiThreadLocking);
ViStatus _VI_FUNC rscmwam_GetError (ViSession instrumentHandle, ViStatus* code,
                                    ViInt32 bufferSize,
                                    ViChar _VI_FAR description[]);
ViStatus _VI_FUNC rscmwam_ClearError (ViSession instrumentHandle);
ViStatus _VI_FUNC rscmwam_ReadToFileFromInstrument (ViSession instrumentHandle,
                                                    ViString source,
                                                    ViString destination);
ViStatus _VI_FUNC rscmwam_WriteFromFileToInstrument (ViSession instrumentHandle,
                                                     ViString source,
                                                     ViString destination);
ViStatus _VI_FUNC rscmwam_WriteInstrData (ViSession instrumentHandle,
                                          ViString writeBuffer);
ViStatus _VI_FUNC rscmwam_ReadInstrData (ViSession instrumentHandle,
                                         ViInt32 numberOfBytesToRead,
                                         ViChar _VI_FAR readBuffer[],
                                         ViUInt32* numBytesRead);
ViStatus _VI_FUNC rscmwam_QueryViBoolean (ViSession instrumentHandle,
                                          ViString command, ViBoolean* value);
ViStatus _VI_FUNC rscmwam_QueryViInt32 (ViSession instrumentHandle,
                                        ViString command, ViInt32* value);
ViStatus _VI_FUNC rscmwam_QueryViReal64 (ViSession instrumentHandle,
                                         ViString command, ViReal64* value);
ViStatus _VI_FUNC rscmwam_QueryViString (ViSession instrumentHandle,
                                         ViString command, ViInt32 bufferSize,
                                         ViChar _VI_FAR value[]);
ViStatus _VI_FUNC rscmwam_close (ViSession instrumentHandle);

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSCMWAM_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSCMWAM_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSCMWAM_ERROR_INCORRECT_STATE        (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSCMWAM_ERROR_INCORRECT_RESPONSE     (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSCMWAM_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)
#define RSCMWAM_ERROR_RELIABILITY_INDICATOR  (RS_SPECIFIC_ERROR_BASE + 0x0004L)

#define RSCMWAM_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSCMWAM_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSCMWAM_ERRMSG_INCORRECT_STATE     "Instrument in incorrect state"
#define RSCMWAM_ERRMSG_INCORRECT_RESPONSE      "Response incorrect; report problem to R&S support!"
#define RSCMWAM_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"
#define RSCMWAM_ERRMSG_RELIABILITY_INDICATOR "Reliability Indicator error"

#define RSCMWAM_ERROR_CODES_AND_MSGS \
    {RSCMWAM_WARN_MEASURE_UNCALIBRATED,  RSCMWAM_WARNMSG_MEASURE_UNCALIBRATED},\
    {RSCMWAM_WARN_OVER_RANGE,            RSCMWAM_WARNMSG_OVER_RANGE},\
    {RSCMWAM_ERROR_INCORRECT_STATE ,   RSCMWAM_ERRMSG_INCORRECT_STATE },\
    {RSCMWAM_ERROR_INCORRECT_RESPONSE,     RSCMWAM_ERRMSG_INCORRECT_RESPONSE},\
    {RSCMWAM_ERROR_MAX_TIME_EXCEEDED,    RSCMWAM_ERRMSG_MAX_TIME_EXCEEDED},\
    {RSCMWAM_ERROR_RELIABILITY_INDICATOR, RSCMWAM_ERRMSG_RELIABILITY_INDICATOR}

/****************************************************************************
 *---------------------------- End Include File ----------------------------*
 ****************************************************************************/

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSCMWAM_HEADER */
