/*****************************************************************************
 *------------------------ Utility  -----------------------------------------*
 *****************************************************************************/
#include <float.h>
#include <math.h>
#include "rscmwam.h"

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/
#pragma warning( disable : 4152 )
/*****************************************************************************
 * Function: rscmwam_InitAttributes
 * Purpose:  This function inits attributes to the desired values if needed.
 *****************************************************************************/
ViStatus rscmwam_InitAttributes (ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    // - Class Driver Identification -
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_CLASS_DRIVER_DESCRIPTION, 0, RSCMWAM_CLASS_DRIVER_DESCRIPTION));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_CLASS_DRIVER_PREFIX, 0, RSCMWAM_CLASS_DRIVER_PREFIX));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_CLASS_DRIVER_VENDOR, 0, RSCMWAM_CLASS_DRIVER_VENDOR));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_CLASS_DRIVER_REVISION, 0, RSCMWAM_CLASS_DRIVER_REVISION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, RSCMWAM_CLASS_SPEC_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, RSCMWAM_CLASS_SPEC_MINOR_VERSION));

    // - Driver Identification -
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION, 0, RSCMWAM_SPECIFIC_DRIVER_DESCRIPTION));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_PREFIX, 0, RSCMWAM_SPECIFIC_DRIVER_PREFIX));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_LOCATOR, 0, RSCMWAM_SPECIFIC_DRIVER_LOCATOR));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_VENDOR, 0, RSCMWAM_SPECIFIC_DRIVER_VENDOR));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, RSCMWAM_CLASS_SPEC_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, RSCMWAM_CLASS_SPEC_MINOR_VERSION));

    // - Driver Capabilities -
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SUPPORTED_INSTRUMENT_MODELS, 0, RSCMWAM_SUPPORTED_INSTRUMENT_MODELS));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_GROUP_CAPABILITIES, 0, RSCMWAM_GROUP_CAPABILITIES));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_FUNCTION_CAPABILITIES, 0, RSCMWAM_FUNCTION_CAPABILITIES));

    // - Version Info -
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION, 0, RSCMWAM_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_VERSION, 0, RSCMWAM_MINOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_MINOR_VERSION, 0, RSCMWAM_MINOR_MINOR_VERSION));

    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_MAJOR_VERSION, 0, RSCMWAM_CLASS_SPEC_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_MINOR_VERSION, 0, RSCMWAM_CLASS_SPEC_MINOR_VERSION));

    checkErr(RsCore_SetSpecificDriverRevision(instrSession, RS_ATTR_SPECIFIC_DRIVER_REVISION));

Error:
    return error;
}

/*****************************************************************************
 * Function: rscmwam_DefaultInstrSetup
 * Purpose:  This function sends a default setup to the instrument. The
 *           rscmwam_reset function calls this function. The
 *           rscmwam_RsInit function calls this function when the
 *           user passes VI_FALSE for the reset parameter. This function is
 *           useful for configuring settings that other instrument driver
 *           functions require.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rscmwam_DefaultInstrSetup (ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));

    // Set all the attributes to the default state. Do not update inherent attributes!
    checkErr(RsCore_ApplyAttributeDefaults(instrSession, VI_FALSE));

    // Init attributes
    checkErr(rscmwam_InitAttributes(instrSession));

    rsSession->fastSweepInstrument = 0;
    checkErr(RsCore_ResetRegistersEseSre(instrSession));
    checkErr(RsCore_Write(instrSession, ":FORM:BASE ASC,10"));
    checkErr(RsCore_Write(instrSession, ":SYST:DISP:UPD OFF"));
    checkErr(RsCore_QueryViInt32(instrSession, "*OPC?", NULL));

Error:
    return error;
}

/*****************************************************************************
 * Function: rscmwam_CheckStatus
 * Purpose:  This function checks the status of the instrument to detect
 *           whether the instrument has encountered an error. This function
 *           is called at the end of most exported driver functions. If the
 *           instrument reports an error, this function returns
 *           RS_ERROR_INSTRUMENT_SPECIFIC. The user can set the
 *           rscmwam_ATTR_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE to disable this
 *           check and increase execution speed.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rscmwam_CheckStatus (ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    if (RsCore_QueryInstrStatus(instrSession) && !RsCore_Simulating(instrSession))
    {
        checkErr(RsCore_CheckStatus(instrSession, VI_SUCCESS));
    }

Error:
    return error;
}

/*****************************************************************************
 * Function: rscmwam_instanceRepCap
 * Purpose:  This function checks instance repeated capability and
 *           creates proper string.
 *****************************************************************************/
ViStatus rscmwam_instanceRepCap (ViSession instrSession, ViInt32 instance, ViChar repCap[])
{
    ViStatus    error   = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
    		2, "Instance");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld", instance);

Error:
    return error;
}

/*****************************************************************************
 * Function: rscmwam_QueryMeasStatus
 * Purpose:  This function queries measurement status
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_QueryMeasStatus(
    ViSession   instrSession,
    ViString    command,
    ViInt32*    mainState,
    ViInt32*    syncState,
    ViInt32*    resourcesState
    )
{
	ViStatus error = VI_SUCCESS;
	ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
	ViChar*  p2buf;
	ViString mainStateArr[] = { "OFF","RDY","RUN",NULL };
	ViString syncStateArr[] = { "PEND","ADJ","INV",NULL };
	ViString resStateArr[] = { "QUE","ACT","INV",NULL };

	checkErr(RsCore_QueryViString(instrSession, command, response));

    p2buf = strtok (response, ",");
    if (p2buf == NULL)
        return RSCMWAM_ERROR_INCORRECT_STATE;
    *mainState = RsCore_FindStringIndex(mainStateArr,p2buf);
    if (*mainState < 0)
        return RSCMWAM_ERROR_INCORRECT_STATE;

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        return RSCMWAM_ERROR_INCORRECT_STATE;
    *syncState = RsCore_FindStringIndex(syncStateArr,p2buf);
    if (*syncState < 0)
        return RSCMWAM_ERROR_INCORRECT_STATE;

    p2buf = strtok (NULL, ",\n");
    if (p2buf == NULL)
        return RSCMWAM_ERROR_INCORRECT_STATE;
    *resourcesState = RsCore_FindStringIndex(resStateArr,p2buf);
    if (*resourcesState < 0)
        return RSCMWAM_ERROR_INCORRECT_STATE;

Error:
    return error;
}

/*****************************************************************************
 * Function: rscmwam_GetMeasTrace
 * Purpose:  This function queries measurement data trace
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_GetMeasTrace(
    ViSession   instrSession,
    ViString    command,
    ViInt32     timeout,
    ViInt32     arraySize,
    ViInt32*    reliability,
    ViReal64    result[],
    ViInt32     *returnedValues
    )
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViInt32  i = 0;

    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, command, timeout, &ext_buf));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWAM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
		*reliability = RsCore_Convert_String_To_ViInt32(p2buf);

    p2buf = strtok (NULL, ",\n");
    while (p2buf != NULL)
    {
        if (i<arraySize)
        {
            result[i++] = RsCore_Convert_String_To_ViReal64(p2buf);
        }
        p2buf = strtok (NULL, ",\n");
    }
    if (returnedValues) *returnedValues=i;

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rscmwam_GetSingleMeasInteger
 * Purpose:  This function queries measurement data
 *****************************************************************************/
ViStatus _VI_FUNC rscmwam_GetSingleMeasInteger(
    ViSession   instrSession,
    ViString    command,
    ViInt32*    reliability,
    ViInt32*    result
    )
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, command, &ext_buf));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWAM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
		*reliability = RsCore_Convert_String_To_ViInt32(p2buf);

    p2buf = strtok (NULL, ",\n");
    *result = RsCore_Convert_String_To_ViInt32(p2buf);

    checkErr(rscmwam_CheckStatus(instrSession));

Error:
    return error;
}